@@ -25,21 +25,27 @@ from salt.exceptions import CommandExecutionError
 
 log = logging.getLogger(__name__)
 
+# Define the module's virtual name
+__virtualname__ = 'partition'
 
 # Define a function alias in order not to shadow built-in's
 __func_alias__ = {
     'set_': 'set',
-    'part_list': 'list'
+    'list_': 'list',
 }
 
 
+VALID_UNITS = set(['s', 'B', 'kB', 'MB', 'MiB', 'GB', 'GiB', 'TB', 'TiB', '%',
+                   'cyl', 'chs', 'compact'])
+
+
 def __virtual__():
     '''
     Only work on POSIX-like systems
     '''
     if salt.utils.is_windows():
         return False
-    return 'partition'
+    return __virtualname__
 
 
 def probe(device=''):
@@ -66,9 +72,7 @@ def probe(device=''):
 
 def part_list(device, unit=None):
     '''
-    partition.part_list device unit
-
-    Prints partition information of given <device>
+    Deprecated. Calls partition.list.
 
     CLI Examples:
 
@@ -78,6 +82,23 @@ def part_list(device, unit=None):
         salt '*' partition.part_list /dev/sda unit=s
         salt '*' partition.part_list /dev/sda unit=kB
     '''
+    return list_(device, unit)
+
+
+def list_(device, unit=None):
+    '''
+    partition.list device unit
+
+    Prints partition information of given <device>
+
+    CLI Examples:
+
+    .. code-block:: bash
+
+        salt '*' partition.list /dev/sda
+        salt '*' partition.list /dev/sda unit=s
+        salt '*' partition.list /dev/sda unit=kB
+    '''
     if device:
         dev = device.replace('/dev/', '')
         if dev not in os.listdir('/dev'):
@@ -86,9 +107,7 @@ def part_list(device, unit=None):
             )
 
     if unit:
-        valid = set('s', 'B', 'kB', 'MB', 'MiB', 'GB', 'GiB', 'TB', 'TiB', '%',
-                    'cyl', 'chs', 'compact')
-        if unit not in valid:
+        if unit not in VALID_UNITS:
             raise CommandExecutionError(
                 'Invalid unit passed to partition.part_list'
             )
@@ -153,7 +172,7 @@ def align_check(device, part_type, partition):
             'Invalid device passed to partition.align_check'
         )
 
-    if part_type not in set('minimal', 'optimal'):
+    if part_type not in set(['minimal', 'optimal']):
         raise CommandExecutionError(
             'Invalid part_type passed to partition.align_check'
         )
@@ -346,8 +365,8 @@ def mkfs(device, fs_type):
     if dev not in os.listdir('/dev'):
         raise CommandExecutionError('Invalid device passed to partition.mkfs')
 
-    if fs_type not in set('ext2', 'fat32', 'fat16', 'linux-swap', 'reiserfs',
-                          'hfs', 'hfs+', 'hfsx', 'NTFS', 'ufs'):
+    if fs_type not in set(['ext2', 'fat32', 'fat16', 'linux-swap', 'reiserfs',
+                          'hfs', 'hfs+', 'hfsx', 'NTFS', 'ufs']):
         raise CommandExecutionError('Invalid fs_type passed to partition.mkfs')
 
     cmd = 'mkfs.{0} {1}'.format(fs_type, device)
@@ -375,8 +394,8 @@ def mklabel(device, label_type):
             'Invalid device passed to partition.mklabel'
         )
 
-    if label_type not in set('aix', 'amiga', 'bsd', 'dvh', 'gpt', 'loop',
-                             'mac', 'msdos', 'pc98', 'sun'):
+    if label_type not in set(['aix', 'amiga', 'bsd', 'dvh', 'gpt', 'loop', 'mac',
+                             'msdos', 'pc98', 'sun']):
         raise CommandExecutionError(
             'Invalid label_type passed to partition.mklabel'
         )
@@ -386,6 +405,18 @@ def mklabel(device, label_type):
     return out
 
 
+def _validate_partition_boundary(boundary):
+    try:
+        for unit in VALID_UNITS:
+            if boundary.endswith(unit):
+                return
+        int(boundary)
+    except Exception:
+        raise CommandExecutionError(
+            'Invalid partition boundary passed: "{0}"'.format(boundary)
+        )
+
+
 def mkpart(device, part_type, fs_type, start, end):
     '''
     partition.mkpart device part_type fs_type start end
@@ -406,24 +437,19 @@ def mkpart(device, part_type, fs_type, start, end):
             'Invalid device passed to partition.mkpart'
         )
 
-    if part_type not in set('primary', 'logical', 'extended'):
+    if part_type not in set(['primary', 'logical', 'extended']):
         raise CommandExecutionError(
             'Invalid part_type passed to partition.mkpart'
         )
 
-    if fs_type not in set('ext2', 'fat32', 'fat16', 'linux-swap', 'reiserfs',
-                          'hfs', 'hfs+', 'hfsx', 'NTFS', 'ufs'):
+    if fs_type not in set(['ext2', 'fat32', 'fat16', 'linux-swap', 'reiserfs',
+                          'hfs', 'hfs+', 'hfsx', 'NTFS', 'ufs']):
         raise CommandExecutionError(
             'Invalid fs_type passed to partition.mkpart'
         )
 
-    try:
-        int(start)
-        int(end)
-    except Exception:
-        raise CommandExecutionError(
-            'Invalid minor partition boundary passed to partition.mkpart'
-        )
+    _validate_partition_boundary(start)
+    _validate_partition_boundary(end)
 
     cmd = 'parted -m -s -- {0} mkpart {1} {2} {3} {4}'.format(
         device, part_type, fs_type, start, end
@@ -454,24 +480,19 @@ def mkpartfs(device, part_type, fs_type, start, end):
             'Invalid device passed to partition.mkpartfs'
         )
 
-    if part_type not in set('primary', 'logical', 'extended'):
+    if part_type not in set(['primary', 'logical', 'extended']):
         raise CommandExecutionError(
             'Invalid part_type passed to partition.mkpartfs'
         )
 
-    if fs_type not in set('ext2', 'fat32', 'fat16', 'linux-swap', 'reiserfs',
-                          'hfs', 'hfs+', 'hfsx', 'NTFS', 'ufs'):
+    if fs_type not in set(['ext2', 'fat32', 'fat16', 'linux-swap', 'reiserfs',
+                          'hfs', 'hfs+', 'hfsx', 'NTFS', 'ufs']):
         raise CommandExecutionError(
             'Invalid fs_type passed to partition.mkpartfs'
         )
 
-    try:
-        int(start)
-        int(end)
-    except Exception:
-        raise CommandExecutionError(
-            'Invalid partition boundary passed to partition.mkpartfs'
-        )
+    _validate_partition_boundary(start)
+    _validate_partition_boundary(end)
 
     cmd = 'parted -m -s -- {0} mkpart {1} {2} {3} {4}'.format(
         device, part_type, fs_type, start, end
@@ -534,13 +555,8 @@ def rescue(device, start, end):
     if dev not in os.listdir('/dev'):
         raise CommandExecutionError('Invalid device passed to partition.rescue')
 
-    try:
-        int(start)
-        int(end)
-    except Exception:
-        raise CommandExecutionError(
-            'Invalid partition boundary passed to partition.rescue'
-        )
+    _validate_partition_boundary(start)
+    _validate_partition_boundary(end)
 
     cmd = 'parted -m -s {0} rescue {1} {2}'.format(device, start, end)
     out = __salt__['cmd.run'](cmd).splitlines()
@@ -574,13 +590,8 @@ def resize(device, minor, start, end):
             'Invalid minor number passed to partition.resize'
         )
 
-    try:
-        int(start)
-        int(end)
-    except Exception:
-        raise CommandExecutionError(
-            'Invalid partition boundary passed to partition.resize'
-        )
+    _validate_partition_boundary(start)
+    _validate_partition_boundary(end)
 
     out = __salt__['cmd.run'](
         'parted -m -s -- {0} resize {1} {2} {3}'.format(
@@ -643,11 +654,11 @@ def set_(device, minor, flag, state):
             'Invalid minor number passed to partition.set'
         )
 
-    if flag not in set('bios_grub', 'legacy_boot', 'boot', 'lba', 'root',
-                       'swap', 'hidden', 'raid', 'LVM', 'PALO', 'PREP', 'DIAG'):
+    if flag not in set(['bios_grub', 'legacy_boot', 'boot', 'lba', 'root',
+                       'swap', 'hidden', 'raid', 'LVM', 'PALO', 'PREP', 'DIAG']):
         raise CommandExecutionError('Invalid flag passed to partition.set')
 
-    if state not in set('on', 'off'):
+    if state not in set(['on', 'off']):
         raise CommandExecutionError('Invalid state passed to partition.set')
 
     cmd = 'parted -m -s {0} set {1} {2} {3}'.format(device, minor, flag, state)
@@ -678,8 +689,8 @@ def toggle(device, partition, flag):
             'Invalid partition number passed to partition.toggle'
         )
 
-    if flag not in set('bios_grub', 'legacy_boot', 'boot', 'lba', 'root',
-                       'swap', 'hidden', 'raid', 'LVM', 'PALO', 'PREP', 'DIAG'):
+    if flag not in set(['bios_grub', 'legacy_boot', 'boot', 'lba', 'root',
+                       'swap', 'hidden', 'raid', 'LVM', 'PALO', 'PREP', 'DIAG']):
         raise CommandExecutionError('Invalid flag passed to partition.toggle')
 
     cmd = 'parted -m -s {0} toggle {1} {2} {3}'.format(device, partition, flag)
