@@ -5,12 +5,11 @@ The networking module for RHEL/Fedora based distros
 
 # Import python libs
 import logging
-import re
 import os.path
 import os
 import StringIO
 
-# import third party libs
+# Import third party libs
 import jinja2
 import jinja2.exceptions
 
@@ -18,23 +17,27 @@ import jinja2.exceptions
 import salt.utils
 import salt.utils.templates
 import salt.utils.validate.net
-from salt.modules import __path__ as saltmodpath
 
 # Set up logging
 log = logging.getLogger(__name__)
 
 # Set up template environment
-ENV = jinja2.Environment(
-    loader=jinja2.FileSystemLoader(saltmodpath[0] + os.sep + 'rh_ip')
+JINJA = jinja2.Environment(
+    loader=jinja2.FileSystemLoader(
+        os.path.join(salt.utils.templates.TEMPLATE_DIRNAME, 'rh_ip')
+    )
 )
 
+# Define the module's virtual name
+__virtualname__ = 'ip'
+
 
 def __virtual__():
     '''
     Confine this module to RHEL/Fedora based distros
     '''
     if __grains__['os_family'] == 'RedHat':
-        return 'ip'
+        return __virtualname__
     return False
 
 
@@ -154,7 +157,7 @@ def _parse_ethtool_opts(opts, iface):
             _raise_error_iface(iface, 'mtu', ['integer'])
 
     if 'speed' in opts:
-        valid = ['10', '100', '1000']
+        valid = ['10', '100', '1000', '10000']
         if str(opts['speed']) in valid:
             config.update({'speed': opts['speed']})
         else:
@@ -374,7 +377,7 @@ def _parse_settings_bond_2(opts, iface, bond_def):
         bond.update({'primary': opts['primary']})
 
     if 'hashing-algorithm' in opts:
-        valid = ['layer2', 'layer3+4']
+        valid = ['layer2', 'layer2+3', 'layer3+4']
         if opts['hashing-algorithm'] in valid:
             bond.update({'xmit_hash_policy': opts['hashing-algorithm']})
         else:
@@ -461,7 +464,7 @@ def _parse_settings_bond_4(opts, iface, bond_def):
         bond.update({'use_carrier': bond_def['use_carrier']})
 
     if 'hashing-algorithm' in opts:
-        valid = ['layer2', 'layer3+4']
+        valid = ['layer2', 'layer2+3', 'layer3+4']
         if opts['hashing-algorithm'] in valid:
             bond.update({'xmit_hash_policy': opts['hashing-algorithm']})
         else:
@@ -611,8 +614,18 @@ def _parse_settings_eth(opts, iface_type, enabled, iface):
         if opt in opts:
             result[opt] = opts[opt]
 
+    for opt in ['ipv6addr', 'ipv6gateway']:
+        if opt in opts:
+            result[opt] = opts[opt]
+
+    if 'ipv6_autoconf' in opts:
+        result['ipv6_autoconf'] = opts['ipv6_autoconf']
+
+    if 'enable_ipv6' in opts:
+        result['enable_ipv6'] = opts['enable_ipv6']
+
     valid = _CONFIG_TRUE + _CONFIG_FALSE
-    for opt in ['peerdns', 'slave', 'vlan', 'defroute']:
+    for opt in ['onparent', 'peerdns', 'slave', 'vlan', 'defroute']:
         if opt in opts:
             if opts[opt] in _CONFIG_TRUE:
                 result[opt] = 'yes'
@@ -656,7 +669,7 @@ def _parse_routes(iface, opts):
     # Normalize keys
     opts = dict((k.lower(), v) for (k, v) in opts.iteritems())
     result = {}
-    if not 'routes' in opts:
+    if 'routes' not in opts:
         _raise_error_routes(iface, 'routes', 'List of routes')
 
     for opt in opts:
@@ -676,7 +689,7 @@ def _parse_network_settings(opts, current):
     result = {}
 
     valid = _CONFIG_TRUE + _CONFIG_FALSE
-    if not 'networking' in opts:
+    if 'networking' not in opts:
         try:
             opts['networking'] = current['networking']
             _log_default_network('networking', current['networking'])
@@ -691,7 +704,7 @@ def _parse_network_settings(opts, current):
     else:
         _raise_error_network('networking', valid)
 
-    if not 'hostname' in opts:
+    if 'hostname' not in opts:
         try:
             opts['hostname'] = current['hostname']
             _log_default_network('hostname', current['hostname'])
@@ -804,12 +817,12 @@ def build_bond(iface, **settings):
 
     opts = _parse_settings_bond(settings, iface)
     try:
-        template = ENV.get_template('conf.jinja')
+        template = JINJA.get_template('conf.jinja')
     except jinja2.exceptions.TemplateNotFound:
         log.error('Could not load template conf.jinja')
         return ''
     data = template.render({'name': iface, 'bonding': opts})
-    _write_file_iface(iface, data, _RH_NETWORK_CONF_FILES, '{0}.conf')
+    _write_file_iface(iface, data, _RH_NETWORK_CONF_FILES, '{0}.conf'.format(iface))
     path = os.path.join(_RH_NETWORK_CONF_FILES, '{0}.conf'.format(iface))
     if rh_major == '5':
         __salt__['cmd.run'](
@@ -864,7 +877,7 @@ def build_interface(iface, iface_type, enabled, **settings):
     if iface_type in ['eth', 'bond', 'bridge', 'slave', 'vlan']:
         opts = _parse_settings_eth(settings, iface_type, enabled, iface)
         try:
-            template = ENV.get_template('rh{0}_eth.jinja'.format(rh_major))
+            template = JINJA.get_template('rh{0}_eth.jinja'.format(rh_major))
         except jinja2.exceptions.TemplateNotFound:
             log.error(
                 'Could not load template rh{0}_eth.jinja'.format(
@@ -874,7 +887,7 @@ def build_interface(iface, iface_type, enabled, **settings):
             return ''
         ifcfg = template.render(opts)
 
-    if settings['test']:
+    if 'test' in settings and settings['test']:
         return _read_temp(ifcfg)
 
     _write_file_iface(iface, ifcfg, _RH_NETWORK_SCRIPT_DIR, 'ifcfg-{0}')
@@ -897,7 +910,7 @@ def build_routes(iface, **settings):
     iface = iface.lower()
     opts = _parse_routes(iface, settings)
     try:
-        template = ENV.get_template('route_eth.jinja')
+        template = JINJA.get_template('route_eth.jinja')
     except jinja2.exceptions.TemplateNotFound:
         log.error(
             'Could not load template route_eth.jinja'
@@ -1011,7 +1024,7 @@ def apply_network_settings(**settings):
 
         salt '*' ip.apply_network_settings
     '''
-    if not 'require_reboot' in settings:
+    if 'require_reboot' not in settings:
         settings['require_reboot'] = False
 
     if settings['require_reboot'] in _CONFIG_TRUE:
@@ -1040,7 +1053,7 @@ def build_network_settings(**settings):
     # Build settings
     opts = _parse_network_settings(settings, current_network_settings)
     try:
-        template = ENV.get_template('network.jinja')
+        template = JINJA.get_template('network.jinja')
     except jinja2.exceptions.TemplateNotFound:
         log.error('Could not load template network.jinja')
         return ''
