@@ -3,9 +3,12 @@
 Display no output.
 '''
 
+# Define the module's virtual name
+__virtualname__ = 'quiet'
+
 
 def __virtual__():
-    return 'quiet'
+    return __virtualname__
 
 
 def output(ret):
