@@ -7,44 +7,61 @@ Render the pillar data
 import os
 import collections
 import logging
+from copy import copy
 
 # Import salt libs
 import salt.loader
 import salt.fileclient
 import salt.minion
 import salt.crypt
+import salt.transport
 from salt._compat import string_types
 from salt.template import compile_template
 from salt.utils.dictupdate import update
 from salt.utils.odict import OrderedDict
 from salt.version import __version__
 
+
 log = logging.getLogger(__name__)
 
 
-def get_pillar(opts, grains, id_, env=None, ext=None):
+def merge_recurse(obj_a, obj_b):
+    copied = copy(obj_a)
+    return update(copied, obj_b)
+
+
+def get_pillar(opts, grains, id_, saltenv=None, ext=None, env=None):
     '''
     Return the correct pillar driver based on the file_client option
     '''
+    if env is not None:
+        salt.utils.warn_until(
+            'Boron',
+            'Passing a salt environment should be done using \'saltenv\' '
+            'not \'env\'. This functionality will be removed in Salt Boron.'
+        )
+        # Backwards compatibility
+        saltenv = env
+
     return {
             'remote': RemotePillar,
             'local': Pillar
-            }.get(opts['file_client'], Pillar)(opts, grains, id_, env, ext)
+            }.get(opts['file_client'], Pillar)(opts, grains, id_, saltenv, ext)
 
 
 class RemotePillar(object):
     '''
     Get the pillar from the master
     '''
-    def __init__(self, opts, grains, id_, env, ext=None):
+    def __init__(self, opts, grains, id_, saltenv, ext=None):
         self.opts = opts
-        self.opts['environment'] = env
+        self.opts['environment'] = saltenv
         self.ext = ext
         self.grains = grains
         self.id_ = id_
         self.serial = salt.payload.Serial(self.opts)
-        self.sreq = salt.payload.SREQ(self.opts['master_uri'])
-        self.auth = salt.crypt.SAuth(opts)
+        self.sreq = salt.transport.Channel.factory(opts)
+        # self.auth = salt.crypt.SAuth(opts)
 
     def compile_pillar(self):
         '''
@@ -52,16 +69,19 @@ class RemotePillar(object):
         '''
         load = {'id': self.id_,
                 'grains': self.grains,
-                'env': self.opts['environment'],
+                'saltenv': self.opts['environment'],
                 'ver': '2',
                 'cmd': '_pillar'}
         if self.ext:
             load['ext'] = self.ext
-        ret = self.sreq.send('aes', self.auth.crypticle.dumps(load), 3, 7200)
-        key = self.auth.get_keys()
-        aes = key.private_decrypt(ret['key'], 4)
-        pcrypt = salt.crypt.Crypticle(self.opts, aes)
-        ret_pillar = pcrypt.loads(ret['pillar'])
+        # ret = self.sreq.send(load, tries=3, timeout=7200)
+        ret_pillar = self.sreq.crypted_transfer_decode_dictentry(load, dictkey='pillar', tries=3, timeout=7200)
+
+        # key = self.auth.get_keys()
+        # aes = key.private_decrypt(ret['key'], 4)
+        # pcrypt = salt.crypt.Crypticle(self.opts, aes)
+        # ret_pillar = pcrypt.loads(ret['pillar'])
+
         if not isinstance(ret_pillar, dict):
             log.error(
                 'Got a bad pillar from master, type {0}, expecting dict: '
@@ -75,23 +95,34 @@ class Pillar(object):
     '''
     Read over the pillar top files and render the pillar data
     '''
-    def __init__(self, opts, grains, id_, env, ext=None):
+    def __init__(self, opts, grains, id_, saltenv, ext=None, functions=None):
         # Store the file_roots path so we can restore later. Issue 5449
         self.actual_file_roots = opts['file_roots']
         # use the local file client
-        self.opts = self.__gen_opts(opts, grains, id_, env, ext)
+        self.opts = self.__gen_opts(opts, grains, id_, saltenv, ext)
         self.client = salt.fileclient.get_file_client(self.opts)
+
         if opts.get('file_client', '') == 'local':
             opts['grains'] = grains
-            self.functions = salt.loader.minion_mods(opts)
+
+        # if we didn't pass in functions, lets load them
+        if functions is None:
+            if opts.get('file_client', '') == 'local':
+                self.functions = salt.loader.minion_mods(opts)
+            else:
+                self.functions = salt.loader.minion_mods(self.opts)
         else:
-            self.functions = salt.loader.minion_mods(self.opts)
+            self.functions = functions
+
         self.matcher = salt.minion.Matcher(self.opts, self.functions)
         self.rend = salt.loader.render(self.opts, self.functions)
         # Fix self.opts['file_roots'] so that ext_pillars know the real
         # location of file_roots. Issue 5951
         ext_pillar_opts = dict(self.opts)
         ext_pillar_opts['file_roots'] = self.actual_file_roots
+        self.merge_strategy = 'smart'
+        if opts.get('pillar_source_merging_strategy'):
+            self.merge_strategy = opts['pillar_source_merging_strategy']
         self.ext_pillars = salt.loader.pillars(ext_pillar_opts, self.functions)
 
     def __valid_ext(self, ext):
@@ -105,10 +136,19 @@ class Pillar(object):
             return {}
         return ext
 
-    def __gen_opts(self, opts_in, grains, id_, env=None, ext=None):
+    def __gen_opts(self, opts_in, grains, id_, saltenv=None, ext=None, env=None):
         '''
         The options need to be altered to conform to the file client
         '''
+        if env is not None:
+            salt.utils.warn_until(
+                'Boron',
+                'Passing a salt environment should be done using \'saltenv\' '
+                'not \'env\'. This functionality will be removed in Salt '
+                'Boron.'
+            )
+            # Backwards compatibility
+            saltenv = env
         opts = dict(opts_in)
         opts['file_roots'] = opts['pillar_roots']
         opts['file_client'] = 'local'
@@ -118,7 +158,7 @@ class Pillar(object):
             opts['grains'] = grains
         opts['id'] = id_
         if 'environment' not in opts:
-            opts['environment'] = env
+            opts['environment'] = saltenv
         if opts['state_top'].startswith('salt://'):
             opts['state_top'] = opts['state_top']
         elif opts['state_top'].startswith('/'):
@@ -164,16 +204,16 @@ class Pillar(object):
                             )
                         ]
             else:
-                for env in self._get_envs():
-                    tops[env].append(
+                for saltenv in self._get_envs():
+                    tops[saltenv].append(
                             compile_template(
                                 self.client.cache_file(
                                     self.opts['state_top'],
-                                    env
+                                    saltenv
                                     ),
                                 self.rend,
                                 self.opts['renderer'],
-                                env=env
+                                saltenv=saltenv
                                 )
                             )
         except Exception as exc:
@@ -182,43 +222,43 @@ class Pillar(object):
                         .format(exc)))
 
         # Search initial top files for includes
-        for env, ctops in tops.items():
+        for saltenv, ctops in tops.items():
             for ctop in ctops:
                 if 'include' not in ctop:
                     continue
                 for sls in ctop['include']:
-                    include[env].append(sls)
+                    include[saltenv].append(sls)
                 ctop.pop('include')
         # Go through the includes and pull out the extra tops and add them
         while include:
             pops = []
-            for env, states in include.items():
-                pops.append(env)
+            for saltenv, states in include.items():
+                pops.append(saltenv)
                 if not states:
                     continue
                 for sls in states:
-                    if sls in done[env]:
+                    if sls in done[saltenv]:
                         continue
                     try:
-                        tops[env].append(
+                        tops[saltenv].append(
                                 compile_template(
                                     self.client.get_state(
                                         sls,
-                                        env
+                                        saltenv
                                         ).get('dest', False),
                                     self.rend,
                                     self.opts['renderer'],
-                                    env=env
+                                    saltenv=saltenv
                                     )
                                 )
                     except Exception as exc:
                         errors.append(
                                 ('Rendering Top file {0} failed, render error'
                                  ':\n{1}').format(sls, exc))
-                    done[env].append(sls)
-            for env in pops:
-                if env in include:
-                    include.pop(env)
+                    done[saltenv].append(sls)
+            for saltenv in pops:
+                if saltenv in include:
+                    include.pop(saltenv)
 
         return tops, errors
 
@@ -230,14 +270,14 @@ class Pillar(object):
         orders = collections.defaultdict(dict)
         for ctops in tops.values():
             for ctop in ctops:
-                for env, targets in ctop.items():
-                    if env == 'include':
+                for saltenv, targets in ctop.items():
+                    if saltenv == 'include':
                         continue
                     for tgt in targets:
                         matches = []
-                        states = set()
-                        orders[env][tgt] = 0
-                        for comp in ctop[env][tgt]:
+                        states = OrderedDict()
+                        orders[saltenv][tgt] = 0
+                        for comp in ctop[saltenv][tgt]:
                             if isinstance(comp, dict):
                                 if 'match' in comp:
                                     matches.append(comp)
@@ -248,11 +288,11 @@ class Pillar(object):
                                             order = int(order)
                                         except ValueError:
                                             order = 0
-                                    orders[env][tgt] = order
+                                    orders[saltenv][tgt] = order
                             if isinstance(comp, string_types):
-                                states.add(comp)
-                        top[env][tgt] = matches
-                        top[env][tgt].extend(list(states))
+                                states[comp] = True
+                        top[saltenv][tgt] = matches
+                        top[saltenv][tgt].extend(list(states.keys()))
         return self.sort_top_targets(top, orders)
 
     def sort_top_targets(self, top, orders):
@@ -260,11 +300,13 @@ class Pillar(object):
         Returns the sorted high data from the merged top files
         '''
         sorted_top = collections.defaultdict(OrderedDict)
-        for env, targets in top.items():
+        # pylint: disable=cell-var-from-loop
+        for saltenv, targets in top.items():
             sorted_targets = sorted(targets.keys(),
-                    key=lambda target: orders[env][target])
+                    key=lambda target: orders[saltenv][target])
             for target in sorted_targets:
-                sorted_top[env][target] = targets[target]
+                sorted_top[saltenv][target] = targets[target]
+        # pylint: enable=cell-var-from-loop
         return sorted_top
 
     def get_top(self):
@@ -272,7 +314,12 @@ class Pillar(object):
         Returns the high data derived from the top file
         '''
         tops, errors = self.get_tops()
-        return self.merge_tops(tops), errors
+        try:
+            merged_tops = self.merge_tops(tops)
+        except TypeError as err:
+            merged_tops = OrderedDict()
+            errors.append('Error encountered while render pillar top file.')
+        return merged_tops, errors
 
     def top_matches(self, top):
         '''
@@ -280,12 +327,12 @@ class Pillar(object):
         that this minion needs to execute.
 
         Returns:
-        {'env': ['state1', 'state2', ...]}
+        {'saltenv': ['state1', 'state2', ...]}
         '''
         matches = {}
-        for env, body in top.items():
+        for saltenv, body in top.items():
             if self.opts['environment']:
-                if env != self.opts['environment']:
+                if saltenv != self.opts['environment']:
                     continue
             for match, data in body.items():
                 if self.matcher.confirm_top(
@@ -293,14 +340,14 @@ class Pillar(object):
                         data,
                         self.opts.get('nodegroups', {}),
                         ):
-                    if env not in matches:
-                        matches[env] = []
+                    if saltenv not in matches:
+                        matches[saltenv] = []
                     for item in data:
                         if isinstance(item, string_types):
-                            matches[env].append(item)
+                            matches[saltenv].append(item)
         return matches
 
-    def render_pstate(self, sls, env, mods, defaults=None):
+    def render_pstate(self, sls, saltenv, mods, defaults=None):
         '''
         Collect a single pillar sls file and render it
         '''
@@ -308,16 +355,24 @@ class Pillar(object):
             defaults = {}
         err = ''
         errors = []
-        fn_ = self.client.get_state(sls, env).get('dest', False)
+        fn_ = self.client.get_state(sls, saltenv).get('dest', False)
         if not fn_:
-            msg = ('Specified SLS {0!r} in environment {1!r} is not'
-                   ' available on the salt master').format(sls, env)
-            log.error(msg)
-            errors.append(msg)
+            if self.opts['pillar_roots'].get(saltenv):
+                msg = ('Specified SLS {0!r} in environment {1!r} is not'
+                       ' available on the salt master').format(sls, saltenv)
+                log.error(msg)
+                errors.append(msg)
+            else:
+                log.debug('Specified SLS {0!r} in environment {1!r} is not'
+                          ' found, which might be due to environment {1!r}'
+                          ' not being present in "pillar_roots" yet!'
+                          .format(sls, saltenv))
+                # return state, mods, errors
+                return None, mods, errors
         state = None
         try:
             state = compile_template(
-                fn_, self.rend, self.opts['renderer'], env, sls, **defaults)
+                fn_, self.rend, self.opts['renderer'], saltenv, sls, **defaults)
         except Exception as exc:
             msg = 'Rendering SLS {0!r} failed, render error:\n{1}'.format(
                 sls, exc
@@ -349,7 +404,7 @@ class Pillar(object):
                             if sub_sls not in mods:
                                 nstate, mods, err = self.render_pstate(
                                         sub_sls,
-                                        env,
+                                        saltenv,
                                         mods,
                                         defaults
                                         )
@@ -369,10 +424,10 @@ class Pillar(object):
         '''
         pillar = {}
         errors = []
-        for env, pstates in matches.items():
+        for saltenv, pstates in matches.items():
             mods = set()
             for sls in pstates:
-                pstate, mods, err = self.render_pstate(sls, env, mods)
+                pstate, mods, err = self.render_pstate(sls, saltenv, mods)
 
                 if err:
                     errors += err
@@ -389,23 +444,54 @@ class Pillar(object):
                             )
                         )
                         continue
-                    pillar.update(pstate)
+                    update(pillar, pstate)
 
         return pillar, errors
 
-    def ext_pillar(self, pillar):
+    def _external_pillar_data(self,
+                             pillar,
+                             val,
+                             pillar_dirs,
+                             key):
         '''
-        Render the external pillar data
+        Builds actual pillar data structure
+        and update
+        the variable ``pillar``
         '''
-        if not 'ext_pillar' in self.opts:
-            return {}
+
+        ext = None
+
+        # try the new interface, which includes the minion ID
+        # as first argument
+        if isinstance(val, dict):
+            ext = self.ext_pillars[key](self.opts['id'], pillar, **val)
+        elif isinstance(val, list):
+            ext = self.ext_pillars[key](self.opts['id'], pillar, *val)
+        else:
+            if key == 'git':
+                ext = self.ext_pillars[key](self.opts['id'],
+                                            val,
+                                            pillar_dirs)
+            else:
+                ext = self.ext_pillars[key](self.opts['id'],
+                                            pillar,
+                                            val)
+        return ext
+
+    def ext_pillar(self, pillar, pillar_dirs):
+        '''
+        Render the external pillar data, and merge with pillar data
+        '''
+        if 'ext_pillar' not in self.opts:
+            return pillar
         if not isinstance(self.opts['ext_pillar'], list):
             log.critical('The "ext_pillar" option is malformed')
-            return {}
+            return pillar
+        ext = None
         for run in self.opts['ext_pillar']:
             if not isinstance(run, dict):
                 log.critical('The "ext_pillar" option is malformed')
-                return {}
+                return pillar
             for key, val in run.items():
                 if key not in self.ext_pillars:
                     err = ('Specified ext_pillar interface {0} is '
@@ -414,32 +500,17 @@ class Pillar(object):
                     continue
                 try:
                     try:
-                        # try the new interface, which includes the minion ID
-                        # as first argument
-                        if isinstance(val, dict):
-                            ext = self.ext_pillars[key](self.opts['id'], pillar, **val)
-                        elif isinstance(val, list):
-                            ext = self.ext_pillars[key](self.opts['id'], pillar, *val)
-                        else:
-                            ext = self.ext_pillars[key](self.opts['id'], pillar, val)
-                        update(pillar, ext)
-
-                    except TypeError as e:
-                        if e.message.startswith('ext_pillar() takes exactly '):
+                        ext = self._external_pillar_data(pillar,
+                                                         val,
+                                                         pillar_dirs,
+                                                         key)
+                    except TypeError as exc:
+                        if exc.message.startswith('ext_pillar() takes exactly '):
                             log.warning('Deprecation warning: ext_pillar "{0}"'
                                         ' needs to accept minion_id as first'
                                         ' argument'.format(key))
                         else:
                             raise
-
-                        if isinstance(val, dict):
-                            ext = self.ext_pillars[key](pillar, **val)
-                        elif isinstance(val, list):
-                            ext = self.ext_pillars[key](pillar, *val)
-                        else:
-                            ext = self.ext_pillars[key](pillar, val)
-                        update(pillar, ext)
-
                 except Exception as exc:
                     log.exception(
                             'Failed to load ext_pillar {0}: {1}'.format(
@@ -447,16 +518,39 @@ class Pillar(object):
                                 exc
                                 )
                             )
+            if ext:
+                pillar = self.merge_sources(pillar, ext)
+                ext = None
         return pillar
 
-    def compile_pillar(self):
+    def merge_sources(self, obj_a, obj_b):
+        strategy = self.merge_strategy
+
+        if strategy == 'smart':
+            renderer = self.opts.get('renderer', 'yaml')
+            if renderer == 'yamlex' or renderer.startswith('yamlex_'):
+                strategy = 'aggregate'
+            else:
+                strategy = 'recurse'
+
+        if strategy == 'recurse':
+            merged = merge_recurse(obj_a, obj_b)
+        else:
+            log.warning('unknown merging strategy {0}, '
+                        'fallback to recurse'.format(strategy))
+            merged = merge_recurse(obj_a, obj_b)
+
+        return merged
+
+    def compile_pillar(self, ext=True, pillar_dirs=None):
         '''
         Render the pillar data and return
         '''
         top, terrors = self.get_top()
         matches = self.top_matches(top)
         pillar, errors = self.render_pillar(matches)
-        self.ext_pillar(pillar)
+        if ext:
+            pillar = self.ext_pillar(pillar, pillar_dirs)
         errors.extend(terrors)
         if self.opts.get('pillar_opts', True):
             mopts = dict(self.opts)
