@@ -23,6 +23,12 @@
 import sys
 import os.path
 
+if 'SETUP_DIRNAME' in globals():
+    # This is from the exec() call in Salt's setup.py
+    THIS_FILE = os.path.join(SETUP_DIRNAME, 'salt', 'syspaths.py')  # pylint: disable=E0602
+else:
+    THIS_FILE = __file__
+
 try:
     # Let's try loading the system paths from the generated module at
     # installation time.
@@ -58,6 +64,6 @@ except ImportError:
     BASE_MASTER_ROOTS_DIR = os.path.join(SRV_ROOT_DIR, 'salt-master')
     LOGS_DIR = os.path.join(ROOT_DIR, 'var', 'log', 'salt')
     PIDFILE_DIR = os.path.join(ROOT_DIR, 'var', 'run')
-    INSTALL_DIR = os.path.dirname(os.path.realpath(__file__))
+    INSTALL_DIR = os.path.dirname(os.path.realpath(THIS_FILE))
     CLOUD_DIR = os.path.join(INSTALL_DIR, 'cloud')
     BOOTSTRAP = os.path.join(CLOUD_DIR, 'deploy', 'bootstrap-salt.sh')
