@@ -8,10 +8,11 @@
     salt.utils.parsers
     ~~~~~~~~~~~~~~~~~~
 
-    This is were all the black magic happens on all of salt's CLI tools.
+    This is where all the black magic happens on all of salt's CLI tools.
 '''
 
 # Import python libs
+from __future__ import print_function
 import os
 import sys
 import getpass
@@ -27,8 +28,55 @@ import salt.utils as utils
 import salt.version as version
 import salt.syspaths as syspaths
 import salt.log.setup as log
-import salt.utils
 from salt.utils.validate.path import is_writeable
+from salt._compat import string_types
+from salt.minion import yamlify_arg
+
+if not utils.is_windows():
+    import salt.cloud.exceptions
+
+# Import 3rd-party libs
+from yaml.scanner import ScannerError as YAMLScannerError
+
+
+def parse_args_kwargs(args):
+    '''
+    Parse out the args and kwargs from an args string
+    '''
+    _args = []
+    _kwargs = {}
+    for arg in args:
+        if isinstance(arg, string_types):
+            arg_name, arg_value = utils.parse_kwarg(arg)
+            if arg_name:
+                if arg_value.strip() == '':
+                    # Because YAML loads empty strings as None, we return the original string
+                    # >>> import yaml
+                    # >>> yaml.load('') is None
+                    # True
+                    # >>> yaml.load('      ') is None
+                    # True
+                    _kwargs[arg_name] = arg_value
+                    continue
+                try:
+                    _kwargs[arg_name] = yamlify_arg(arg_value)
+                except YAMLScannerError:
+                    _kwargs[arg_name] = arg_value
+            else:
+                if arg.strip() == '':
+                    # Because YAML loads empty strings as None, we return the original string
+                    # >>> import yaml
+                    # >>> yaml.load('') is None
+                    # True
+                    # >>> yaml.load('      ') is None
+                    # True
+                    _args.append(arg)
+                    continue
+                try:
+                    _args.append(yamlify_arg(arg))
+                except YAMLScannerError:
+                    _args.append(arg)
+    return _args, _kwargs
 
 
 def _sorted(mixins_or_funcs):
@@ -94,7 +142,7 @@ class OptionParserMeta(MixInMeta):
 
 
 class OptionParser(optparse.OptionParser):
-    VERSION = version.__version__
+    VERSION = version.__saltstack_version__.formatted_version
 
     usage = '%prog'
 
@@ -189,7 +237,7 @@ class OptionParser(optparse.OptionParser):
         )
 
     def print_versions_report(self, file=sys.stdout):
-        print >> file, '\n'.join(version.versions_report())
+        print('\n'.join(version.versions_report()), file=file)
         self.exit()
 
 
@@ -275,8 +323,11 @@ class ConfigDirMixIn(object):
     _config_filename_ = None
 
     def _mixin_setup(self):
+        config_dir = os.environ.get('SALT_CONFIG_DIR', None)
+        if not config_dir:
+            config_dir = syspaths.CONFIG_DIR
         self.add_option(
-            '-c', '--config-dir', default=syspaths.CONFIG_DIR,
+            '-c', '--config-dir', default=config_dir,
             help=('Pass in an alternative configuration directory. Default: '
                   '%default')
         )
@@ -294,7 +345,12 @@ class ConfigDirMixIn(object):
         self.options.config_dir = os.path.abspath(self.options.config_dir)
 
         if hasattr(self, 'setup_config'):
-            self.config = self.setup_config()
+            try:
+                self.config = self.setup_config()
+            except (IOError, OSError) as exc:
+                self.error(
+                    'Failed to load configuration: {0}'.format(exc)
+                )
 
     def get_config_file_path(self, configfile=None):
         if configfile is None:
@@ -526,7 +582,7 @@ class LogLevelMixIn(object):
             # Since we're not be able to write to the log file or it's parent
             # directory(if the log file does not exit), are we the same user
             # as the one defined in the configuration file?
-            current_user = getpass.getuser()
+            current_user = salt.utils.get_user()
             if self.config['user'] != current_user:
                 # Yep, not the same user!
                 # Is the current user in ACL?
@@ -771,13 +827,6 @@ class ExtendedTargetOptionsMixIn(TargetOptionsMixIn):
                   'matching the specific targets argument type: salt '
                   '\'G@os:RedHat and webser* or E@database.*\'')
         )
-        group.add_option(
-            '-X', '--exsel',
-            default=False,
-            action='store_true',
-            help=('Instead of using shell globs use the return code of '
-                  'a function.')
-        )
         group.add_option(
             '-I', '--pillar',
             default=False,
@@ -928,7 +977,7 @@ class OutputOptionsWithTextMixIn(OutputOptionsMixIn):
             cls, *args, **kwargs
         )
         utils.warn_until(
-            (0, 19),
+            'Helium',
             '\'OutputOptionsWithTextMixIn\' has been deprecated. Please '
             'start using \'OutputOptionsMixIn\'; your code should not need '
             'any further changes.'
@@ -936,6 +985,385 @@ class OutputOptionsWithTextMixIn(OutputOptionsMixIn):
         return instance
 
 
+class CloudConfigMixIn(object):
+    __metaclass__ = MixInMeta
+    _mixin_prio_ = -11    # Evaluate before ConfigDirMixin
+
+    def _mixin_setup(self):
+        group = self.config_group = optparse.OptionGroup(
+            self,
+            'Configuration Options',
+            # Include description here as a string
+        )
+        group.add_option(
+            '-C', '--cloud-config',
+            default=None,
+            help='DEPRECATED. The location of the salt-cloud config file.'
+        )
+        group.add_option(
+            '-M', '--master-config',
+            default=None,
+            help='DEPRECATED. The location of the salt master config file.'
+        )
+        group.add_option(
+            '-V', '--profiles', '--vm_config',
+            dest='vm_config',
+            default=None,
+            help='DEPRECATED. The location of the salt.cloud VM config file.'
+        )
+        group.add_option(
+            '--providers-config',
+            default=None,
+            help='DEPRECATED. The location of the salt cloud VM providers '
+                 'configuration file.'
+        )
+        self.add_option_group(group)
+
+    def __assure_absolute_paths(self, name):
+        # Need to check if file exists?
+        optvalue = getattr(self.options, name)
+        if optvalue:
+            setattr(self.options, name, os.path.abspath(optvalue))
+
+    def _mixin_after_parsed(self):
+        for option in self.config_group.option_list:
+            if option.dest is None:
+                # This should not happen.
+                #
+                # --version does not have dest attribute set for example.
+                # All options defined by us, even if not explicitly(by kwarg),
+                # will have the dest attribute set
+                continue
+            self.__assure_absolute_paths(option.dest)
+
+        # Grab data from the 4 sources (done in self.process_cloud_config)
+        # 1st - Master config
+        # 2nd - Override master config with salt-cloud config
+        # 3rd - Include Cloud Providers
+        # 4th - Include VM config
+        # 5th - Override config with cli options
+        # Done in parsers.MergeConfigMixIn.__merge_config_with_cli()
+
+        # Remove log_level_logfile from config if set to None so it can be
+        # equal to console log_level
+        if self.config['log_level_logfile'] is None:
+            self.config.pop('log_level_logfile')
+
+    def process_cloud_config(self):
+        if self.options.cloud_config is not None:
+
+            utils.warn_until(
+                'Helium',
+                'Don\'t forget to remove this support in Helium',
+                _dont_call_warnings=True
+            )
+            logging.getLogger(__name__).info(
+                'Passing \'--cloud-config\' has been deprecated. Instead, store '
+                'all of the salt cloud related configuration files in a single '
+                'directory and pass that directory to \'--config-dir\'. This '
+                'support will be removed in Salt Helium. Note that the '
+                '\'SALT_CLOUD_CONFIG\' environment variable is still valid.'
+            )
+
+    def process_vm_config(self):
+        if self.options.vm_config is not None:
+            utils.warn_until(
+                'Helium',
+                'Don\'t forget to remove this support in Helium',
+                _dont_call_warnings=True
+            )
+            logging.getLogger(__name__).info(
+                'Passing \'--vm_config\' has been deprecated. Instead, store all '
+                'of the salt cloud related configuration files in a single '
+                'directory and pass that directory to \'--config-dir\'. This '
+                'support will be removed in Salt Helium. Note that the '
+                '\'SALT_CLOUDVM_CONFIG\' environment variable is still valid and '
+                'you can also set an absolute path to this setting on the main '
+                'cloud configuration file under \'vm_config\'.'
+            )
+
+    def process_providers_config(self):
+        if self.options.providers_config is not None:
+            utils.warn_until(
+                'Helium',
+                'Don\'t forget to remove this support in Helium',
+                _dont_call_warnings=True
+            )
+            logging.getLogger(__name__).info(
+                'Passing \'--providers-config\' has been deprecated. Instead, '
+                'store all of the salt cloud related configuration files in a '
+                'single directory and pass that directory to \'--config-dir\'. '
+                'This support will be removed in Salt Helium. Note that the '
+                '\'SALT_CLOUD_PROVIDERS_CONFIG\' environment variable is still '
+                'valid and you can also set an absolute path to this setting on '
+                'the main cloud configuration file under \'providers_config\'.'
+            )
+
+    def process_master_config(self):
+        if self.options.master_config is not None:
+            utils.warn_until(
+                'Helium',
+                'Don\'t forget to remove this support in Helium',
+                _dont_call_warnings=True
+            )
+            logging.getLogger(__name__).info(
+                'Passing \'--master-config\' has been deprecated. Instead, store '
+                'all of the salt cloud related configuration files in a single '
+                'directory and pass that directory to \'--config-dir\'. This '
+                'support will be removed in Salt Helium. Note that the '
+                '\'SALT_MASTER_CONFIG\' environment variable is still valid and '
+                'you can also set an absolute path to this setting on the main '
+                'cloud configuration file under \'providers_config\'.'
+            )
+
+
+class ExecutionOptionsMixIn(object):
+    __metaclass__ = MixInMeta
+    _mixin_prio_ = 10
+
+    def _mixin_setup(self):
+        group = self.execution_group = optparse.OptionGroup(
+            self,
+            'Execution Options',
+            # Include description here as a string
+        )
+        group.add_option(
+            '-L', '--location',
+            default=None,
+            help='Specify which region to connect to.'
+        )
+        group.add_option(
+            '-a', '--action',
+            default=None,
+            help='Perform an action that may be specific to this cloud '
+                 'provider. This argument requires one or more instance '
+                 'names to be specified.'
+        )
+        group.add_option(
+            '-f', '--function',
+            nargs=2,
+            default=None,
+            metavar='<FUNC-NAME> <PROVIDER>',
+            help='Perform an function that may be specific to this cloud '
+                 'provider, that does not apply to an instance. This '
+                 'argument requires a provider to be specified (i.e.: nova).'
+        )
+        group.add_option(
+            '-p', '--profile',
+            default=None,
+            help='Create an instance using the specified profile.'
+        )
+        group.add_option(
+            '-m', '--map',
+            default=None,
+            help='Specify a cloud map file to use for deployment. This option '
+                 'may be used alone, or in conjunction with -Q, -F, -S or -d.'
+        )
+        group.add_option(
+            '-H', '--hard',
+            default=False,
+            action='store_true',
+            help='Delete all VMs that are not defined in the map file. '
+                 'CAUTION!!! This operation can irrevocably destroy VMs! It '
+                 'must be explicitly enabled in the cloud config file.'
+        )
+        group.add_option(
+            '-d', '--destroy',
+            default=False,
+            action='store_true',
+            help='Destroy the specified instance(s).'
+        )
+        group.add_option(
+            '--no-deploy',
+            default=True,
+            dest='deploy',
+            action='store_false',
+            help='Don\'t run a deploy script after instance creation.'
+        )
+        group.add_option(
+            '-P', '--parallel',
+            default=False,
+            action='store_true',
+            help='Build all of the specified instances in parallel.'
+        )
+        group.add_option(
+            '-u', '--update-bootstrap',
+            default=False,
+            action='store_true',
+            help='Update salt-bootstrap to the latest develop version on '
+                 'GitHub.'
+        )
+        group.add_option(
+            '-y', '--assume-yes',
+            default=False,
+            action='store_true',
+            help='Default yes in answer to all confirmation questions.'
+        )
+        group.add_option(
+            '-k', '--keep-tmp',
+            default=False,
+            action='store_true',
+            help='Do not remove files from /tmp/ after deploy.sh finishes.'
+        )
+        group.add_option(
+            '--show-deploy-args',
+            default=False,
+            action='store_true',
+            help='Include the options used to deploy the minion in the data '
+                 'returned.'
+        )
+        group.add_option(
+            '--script-args',
+            default=None,
+            help='Script arguments to be fed to the bootstrap script when '
+                 'deploying the VM'
+        )
+        self.add_option_group(group)
+
+    def process_function(self):
+        if self.options.function:
+            self.function_name, self.function_provider = self.options.function
+            if self.function_provider.startswith('-') or \
+                    '=' in self.function_provider:
+                self.error(
+                    '--function expects two arguments: <function-name> '
+                    '<provider>'
+                )
+
+
+class CloudQueriesMixIn(object):
+    __metaclass__ = MixInMeta
+    _mixin_prio_ = 20
+
+    selected_query_option = None
+
+    def _mixin_setup(self):
+        group = self.cloud_queries_group = optparse.OptionGroup(
+            self,
+            'Query Options',
+            # Include description here as a string
+        )
+        group.add_option(
+            '-Q', '--query',
+            default=False,
+            action='store_true',
+            help=('Execute a query and return some information about the '
+                  'nodes running on configured cloud providers')
+        )
+        group.add_option(
+            '-F', '--full-query',
+            default=False,
+            action='store_true',
+            help=('Execute a query and return all information about the '
+                  'nodes running on configured cloud providers')
+        )
+        group.add_option(
+            '-S', '--select-query',
+            default=False,
+            action='store_true',
+            help=('Execute a query and return select information about '
+                  'the nodes running on configured cloud providers')
+        )
+        group.add_option(
+            '--list-providers',
+            default=False,
+            action='store_true',
+            help=('Display a list of configured providers.')
+        )
+        self.add_option_group(group)
+        self._create_process_functions()
+
+    def _create_process_functions(self):
+        for option in self.cloud_queries_group.option_list:
+            def process(opt):
+                if getattr(self.options, opt.dest):
+                    query = 'list_nodes'
+                    if opt.dest == 'full_query':
+                        query += '_full'
+                    elif opt.dest == 'select_query':
+                        query += '_select'
+                    elif opt.dest == 'list_providers':
+                        query = 'list_providers'
+                        if self.args:
+                            self.error(
+                                '\'--list-providers\' does not accept any '
+                                'arguments'
+                            )
+                    self.selected_query_option = query
+
+            funcname = 'process_{0}'.format(option.dest)
+            if not hasattr(self, funcname):
+                setattr(self, funcname, partial(process, option))
+
+    def _mixin_after_parsed(self):
+        group_options_selected = filter(
+            lambda option: getattr(self.options, option.dest) is not False,
+            self.cloud_queries_group.option_list
+        )
+        if len(group_options_selected) > 1:
+            self.error(
+                'The options {0} are mutually exclusive. Please only choose '
+                'one of them'.format('/'.join([
+                    option.get_opt_string() for option in
+                    group_options_selected
+                ]))
+            )
+        self.config['selected_query_option'] = self.selected_query_option
+
+
+class CloudProvidersListsMixIn(object):
+    __metaclass__ = MixInMeta
+    _mixin_prio_ = 30
+
+    def _mixin_setup(self):
+        group = self.providers_listings_group = optparse.OptionGroup(
+            self,
+            'Cloud Providers Listings',
+            # Include description here as a string
+        )
+        group.add_option(
+            '--list-locations',
+            default=None,
+            help=('Display a list of locations available in configured cloud '
+                  'providers. Pass the cloud provider that available '
+                  'locations are desired on, aka "linode", or pass "all" to '
+                  'list locations for all configured cloud providers')
+        )
+        group.add_option(
+            '--list-images',
+            default=None,
+            help=('Display a list of images available in configured cloud '
+                  'providers. Pass the cloud provider that available images '
+                  'are desired on, aka "linode", or pass "all" to list images '
+                  'for all configured cloud providers')
+        )
+        group.add_option(
+            '--list-sizes',
+            default=None,
+            help=('Display a list of sizes available in configured cloud '
+                  'providers. Pass the cloud provider that available sizes '
+                  'are desired on, aka "AWS", or pass "all" to list sizes '
+                  'for all configured cloud providers')
+        )
+        self.add_option_group(group)
+
+    def _mixin_after_parsed(self):
+        list_options_selected = filter(
+            lambda option: getattr(self.options, option.dest) is not None,
+            self.providers_listings_group.option_list
+        )
+        if len(list_options_selected) > 1:
+            self.error(
+                'The options {0} are mutually exclusive. Please only choose '
+                'one of them'.format(
+                    '/'.join([
+                        option.get_opt_string() for option in
+                        list_options_selected
+                    ])
+                )
+            )
+
+
 class MasterOptionParser(OptionParser, ConfigDirMixIn, MergeConfigMixIn,
                          LogLevelMixIn, RunUserMixin, DaemonMixIn,
                          PidfileMixin):
@@ -1063,10 +1491,10 @@ class SaltCMDOptionParser(OptionParser, ConfigDirMixIn, MergeConfigMixIn,
                   'minions to have running')
         )
         self.add_option(
-            '-a', '--auth', '--eauth', '--extended-auth',
+            '-a', '--auth', '--eauth', '--external-auth',
             default='',
             dest='eauth',
-            help=('Specify an extended authentication system to use.')
+            help=('Specify an external authentication system to use.')
         )
         self.add_option(
             '-T', '--make-token',
@@ -1109,11 +1537,11 @@ class SaltCMDOptionParser(OptionParser, ConfigDirMixIn, MergeConfigMixIn,
             try:
                 self.print_help()
             except Exception:
-                        # We get an argument that Python's optparser just can't
-                        # deal with.  Perhaps stdout was redirected, or a file
-                        # glob was passed in. Regardless, we're in an unknown
-                        # state here.
-                sys.stdout.write("Invalid options passed. Please try -h for help.")  # Try to warn if we can.
+                # We get an argument that Python's optparser just can't deal
+                # with. Perhaps stdout was redirected, or a file glob was
+                # passed in. Regardless, we're in an unknown state here.
+                sys.stdout.write('Invalid options passed. Please try -h for '
+                                 'help.')  # Try to warn if we can.
                 sys.exit(1)
 
         if self.options.doc:
@@ -1171,14 +1599,16 @@ class SaltCMDOptionParser(OptionParser, ConfigDirMixIn, MergeConfigMixIn,
                                 self.config['arg'][cmd_index].append(arg)
                         if len(self.config['fun']) != len(self.config['arg']):
                             self.exit(42, 'Cannot execute compound command without '
-                                          'defining all arguments.')
+                                          'defining all arguments.\n')
+                else:
+                    self.config['fun'] = self.args[1]
+                    self.config['arg'] = self.args[2:]
+
+                # parse the args and kwargs before sending to the publish interface
+                self.config['arg'] = salt.client.condition_kwarg(*parse_args_kwargs(self.config['arg']))
             except IndexError:
                 self.exit(42, '\nIncomplete options passed.\n\n')
 
-            else:
-                self.config['fun'] = self.args[1]
-                self.config['arg'] = self.args[2:]
-
     def setup_config(self):
         return config.client_config(self.get_config_file_path())
 
@@ -1243,16 +1673,6 @@ class SaltKeyOptionParser(OptionParser, ConfigDirMixIn, MergeConfigMixIn,
     _default_logging_logfile_ = os.path.join(syspaths.LOGS_DIR, 'key')
 
     def _mixin_setup(self):
-        # XXX: Remove '--key-logfile' support in 0.18.0
-        utils.warn_until((0, 18), '', _dont_call_warnings=True)
-        self.logging_options_group.add_option(
-            '--key-logfile',
-            default=None,
-            help='Send all output to a file. Default is {0!r}'.format(
-                self._default_logging_logfile_
-            )
-        )
-
         actions_group = optparse.OptionGroup(self, 'Actions')
         actions_group.add_option(
             '-l', '--list',
@@ -1270,14 +1690,15 @@ class SaltKeyOptionParser(OptionParser, ConfigDirMixIn, MergeConfigMixIn,
             '-L', '--list-all',
             default=False,
             action='store_true',
-            help='List all public keys. Deprecated: use "--list all"'
+            help='List all public keys. (Deprecated: use "--list all")'
         )
 
         actions_group.add_option(
             '-a', '--accept',
             default='',
             help='Accept the specified public key (use --include-all to '
-                 'match rejected keys in addition to pending keys)'
+                 'match rejected keys in addition to pending keys). Globs are '
+                 'supported.'
         )
 
         actions_group.add_option(
@@ -1291,7 +1712,8 @@ class SaltKeyOptionParser(OptionParser, ConfigDirMixIn, MergeConfigMixIn,
             '-r', '--reject',
             default='',
             help='Reject the specified public key (use --include-all to '
-                 'match accepted keys in addition to pending keys)'
+                 'match accepted keys in addition to pending keys). Globs are '
+                 'supported.'
         )
 
         actions_group.add_option(
@@ -1324,7 +1746,7 @@ class SaltKeyOptionParser(OptionParser, ConfigDirMixIn, MergeConfigMixIn,
         actions_group.add_option(
             '-d', '--delete',
             default='',
-            help='Delete the named key'
+            help='Delete the specified key. Globs are supported.'
         )
 
         actions_group.add_option(
@@ -1337,14 +1759,14 @@ class SaltKeyOptionParser(OptionParser, ConfigDirMixIn, MergeConfigMixIn,
         actions_group.add_option(
             '-f', '--finger',
             default='',
-            help='Print the named key\'s fingerprint'
+            help='Print the specified key\'s fingerprint'
         )
 
         actions_group.add_option(
             '-F', '--finger-all',
             default=False,
             action='store_true',
-            help='Print all key\'s fingerprints'
+            help='Print all keys\' fingerprints'
         )
         self.add_option_group(actions_group)
 
@@ -1414,6 +1836,17 @@ class SaltKeyOptionParser(OptionParser, ConfigDirMixIn, MergeConfigMixIn,
 
         return keys_config
 
+    def process_list(self):
+        # Filter accepted list arguments as soon as possible
+        if not self.options.list:
+            return
+        if not self.options.list.startswith(('acc', 'pre', 'un', 'rej', 'all')):
+            self.error(
+                '{0!r} is not a valid argument to \'--list\''.format(
+                    self.options.list
+                )
+            )
+
     def process_keysize(self):
         if self.options.keysize < 2048:
             self.error('The minimum value for keysize is 2048')
@@ -1425,16 +1858,6 @@ class SaltKeyOptionParser(OptionParser, ConfigDirMixIn, MergeConfigMixIn,
         # --create-keys-dir
         self._mixin_after_parsed_funcs.append(self.__create_keys_dir)
 
-    def process_key_logfile(self):
-        if self.options.key_logfile:
-            # XXX: Remove '--key-logfile' support in 0.18.0
-            # In < 0.18.0 error out
-            utils.warn_until((0, 18), '', _dont_call_warnings=True)
-            self.error(
-                'The \'--key-logfile\' option has been deprecated in favour '
-                'of \'--log-file\''
-            )
-
     def _mixin_after_parsed(self):
         # It was decided to always set this to info, since it really all is
         # info or error.
@@ -1508,6 +1931,16 @@ class SaltCallOptionParser(OptionParser, ConfigDirMixIn, MergeConfigMixIn,
             action='store_true',
             help='Run salt-call locally, as if there was no master running.'
         )
+        self.add_option(
+            '--file-root',
+            default=None,
+            help='Set this directory as the base file root.'
+        )
+        self.add_option(
+            '--pillar-root',
+            default=None,
+            help='Set this directory as the base pillar root.'
+        )
         self.add_option(
             '--retcode-passthrough',
             default=False,
@@ -1522,6 +1955,18 @@ class SaltCallOptionParser(OptionParser, ConfigDirMixIn, MergeConfigMixIn,
             help=('Specify the minion id to use. If this option is omitted, '
                   'the id option from the minion config will be used.')
         )
+        self.add_option(
+            '--skip-grains',
+            default=False,
+            action='store_true',
+            help=('Do not load grains.')
+        )
+        self.add_option(
+            '--refresh-grains-cache',
+            default=False,
+            action='store_true',
+            help=('Force a refresh of the grains cache')
+    )
 
     def _mixin_after_parsed(self):
         if not self.args and not self.options.grains_run \
@@ -1574,8 +2019,9 @@ class SaltRunOptionParser(OptionParser, ConfigDirMixIn, MergeConfigMixIn,
             dest='doc',
             default=False,
             action='store_true',
-            help=('Display documentation for runners, pass a module or a '
-                  'runner to see documentation on only that module/runner.')
+            help=('Display documentation for runners, pass a runner or '
+                  'runner.function to see documentation on only that runner '
+                  'or function.')
         )
 
     def _mixin_after_parsed(self):
@@ -1624,7 +2070,17 @@ class SaltSSHOptionParser(OptionParser, ConfigDirMixIn, MergeConfigMixIn,
             '--roster',
             dest='roster',
             default='',
-            help=('Define which roster system to use'))
+            help=('Define which roster system to use, this defines if a '
+                  'database backend, scanner, or custom roster system is '
+                  'used. Default is the flat file roster.'))
+        self.add_option(
+            '--roster-file',
+            dest='roster_file',
+            default='',
+            help=('define an alternative location for the default roster '
+                  'file location. The default roster file is called roster '
+                  'and is found in the same directory as the master config '
+                  'file.'))
         self.add_option(
             '--refresh', '--refresh-cache',
             dest='refresh_cache',
@@ -1651,12 +2107,27 @@ class SaltSSHOptionParser(OptionParser, ConfigDirMixIn, MergeConfigMixIn,
             action='store_true',
             help='By default ssh host keys are honored and connections will '
                  'ask for approval')
+        self.add_option(
+            '-v', '--verbose',
+            default=False,
+            action='store_true',
+            help=('Turn on command verbosity, display jid')
+        )
         self.add_option(
             '--passwd',
             dest='ssh_passwd',
             default='',
             help='Set the default password to attempt to use when '
-                 'authenticating')
+                 'authenticating'
+        )
+        self.add_option(
+            '--askpass',
+            dest='ssh_askpass',
+            default=False,
+            action='store_true',
+            help='Interactively ask for the SSH password with no echo - avoids '
+                 'password in process args and stored in history'
+        )
         self.add_option(
             '--key-deploy',
             dest='ssh_key_deploy',
@@ -1681,5 +2152,57 @@ class SaltSSHOptionParser(OptionParser, ConfigDirMixIn, MergeConfigMixIn,
         if len(self.args) > 0:
             self.config['arg_str'] = ' '.join(self.args[1:])
 
+        if self.options.ssh_askpass:
+            self.options.ssh_passwd = getpass.getpass('Password: ')
+
     def setup_config(self):
         return config.master_config(self.get_config_file_path())
+
+
+class SaltCloudParser(OptionParser,
+                      LogLevelMixIn,
+                      MergeConfigMixIn,
+                      OutputOptionsMixIn,
+                      ConfigDirMixIn,
+                      CloudConfigMixIn,
+                      CloudQueriesMixIn,
+                      ExecutionOptionsMixIn,
+                      CloudProvidersListsMixIn):
+
+    __metaclass__ = OptionParserMeta
+
+    # ConfigDirMixIn attributes
+    _config_filename_ = 'cloud'
+
+    # LogLevelMixIn attributes
+    _default_logging_level_ = 'info'
+    _logfile_config_setting_name_ = 'log_file'
+    _loglevel_config_setting_name_ = 'log_level_logfile'
+    _default_logging_logfile_ = os.path.join(syspaths.LOGS_DIR, 'cloud')
+
+    def print_versions_report(self, file=sys.stdout):
+        print('\n'.join(version.versions_report(include_salt_cloud=True)),
+              file=file)
+        self.exit()
+
+    def _mixin_after_parsed(self):
+        if 'DUMP_SALT_CLOUD_CONFIG' in os.environ:
+            import pprint
+
+            print('Salt cloud configuration dump(INCLUDES SENSIBLE DATA):')
+            pprint.pprint(self.config)
+            self.exit(0)
+
+        if self.args:
+            self.config['names'] = self.args
+
+    def setup_config(self):
+        try:
+            return config.cloud_config(
+                self.options.cloud_config or self.get_config_file_path(),
+                master_config_path=self.options.master_config,
+                providers_config_path=self.options.providers_config,
+                profiles_config_path=self.options.vm_config
+            )
+        except salt.cloud.exceptions.SaltCloudConfigError as exc:
+            self.error(exc)
