@@ -1,12 +1,16 @@
+# -*- coding: utf-8 -*-
+
 '''
 Set up the Salt integration test suite
 '''
 
 # Import Python libs
-import re
+from __future__ import print_function
 import os
+import re
 import sys
 import time
+import errno
 import shutil
 import pprint
 import logging
@@ -21,6 +25,10 @@ except ImportError:
     pass
 
 
+STATE_FUNCTION_RUNNING_RE = re.compile(
+    r'''The function (?:"|')(?P<state_func>.*)(?:"|') is running as PID '''
+    r'(?P<pid>[\d]+) and was started at (?P<date>.*) with jid (?P<jid>[\d]+)'
+)
 INTEGRATION_TEST_DIR = os.path.dirname(
     os.path.normpath(os.path.abspath(__file__))
 )
@@ -39,12 +47,14 @@ ensure_in_syspath(CODE_DIR, SALT_LIBS)
 
 # Import Salt libs
 import salt
+import salt._compat
 import salt.config
 import salt.master
 import salt.minion
 import salt.runner
 import salt.output
 import salt.version
+import salt.utils
 from salt.utils import fopen, get_colors
 from salt.utils.verify import verify_env
 
@@ -55,21 +65,55 @@ import yaml
 SYS_TMP_DIR = os.environ.get('TMPDIR', tempfile.gettempdir())
 TMP = os.path.join(SYS_TMP_DIR, 'salt-tests-tmpdir')
 FILES = os.path.join(INTEGRATION_TEST_DIR, 'files')
-PYEXEC = 'python{0}.{1}'.format(sys.version_info[0], sys.version_info[1])
+PYEXEC = 'python{0}.{1}'.format(*sys.version_info)
 MOCKBIN = os.path.join(INTEGRATION_TEST_DIR, 'mockbin')
 SCRIPT_DIR = os.path.join(CODE_DIR, 'scripts')
 TMP_STATE_TREE = os.path.join(SYS_TMP_DIR, 'salt-temp-state-tree')
+TMP_PRODENV_STATE_TREE = os.path.join(SYS_TMP_DIR, 'salt-temp-prodenv-state-tree')
 TMP_CONF_DIR = os.path.join(TMP, 'config')
 
 log = logging.getLogger(__name__)
 
 
-def run_tests(TestCase, needs_daemon=True):
+def skip_if_binaries_missing(binaries, check_all=False):
+    # While there's no new release of salt-testing
+    def _id(obj):
+        return obj
+
+    if sys.version_info < (2, 7):
+        from unittest2 import skip  # pylint: disable=F0401
+    else:
+        from unittest import skip  # pylint: disable=E0611
+
+    if check_all:
+        for binary in binaries:
+            if salt.utils.which(binary) is None:
+                return skip(
+                    'The {0!r} binary was not found'
+                )
+    elif salt.utils.which_bin(binaries) is None:
+        return skip(
+            'None of the following binaries was found: {0}'.format(
+                ', '.join(binaries)
+            )
+        )
+    return _id
+
+
+def run_tests(*test_cases, **kwargs):
     '''
-    Run integration tests for a chosen test case.
+    Run integration tests for the chosen test cases.
 
     Function uses optparse to set up test environment
     '''
+
+    needs_daemon = kwargs.pop('needs_daemon', True)
+    if kwargs:
+        raise RuntimeError(
+            'The \'run_tests\' function only accepts \'needs_daemon\' as a '
+            'keyword argument'
+        )
+
     class TestcaseParser(SaltTestcaseParser):
         def setup_additional_options(self):
             self.add_option(
@@ -86,17 +130,18 @@ def run_tests(TestCase, needs_daemon=True):
                 help='Disable colour printing.'
             )
 
-        def run_testcase(self, testcase, needs_daemon=True):
+        def run_testcase(self, testcase, needs_daemon=True):  # pylint: disable=W0221
             if needs_daemon:
-                print('Setting up Salt daemons to execute tests')
+                print(' * Setting up Salt daemons to execute tests')
                 with TestDaemon(self):
                     return SaltTestcaseParser.run_testcase(self, testcase)
             return SaltTestcaseParser.run_testcase(self, testcase)
 
     parser = TestcaseParser()
     parser.parse_args()
-    if parser.run_testcase(TestCase, needs_daemon=needs_daemon) is False:
-        parser.finalize(1)
+    for case in test_cases:
+        if parser.run_testcase(case, needs_daemon=needs_daemon) is False:
+            parser.finalize(1)
     parser.finalize(0)
 
 
@@ -156,6 +201,11 @@ class TestDaemon(object):
                 # Let's support runtime created files that can be used like:
                 #   salt://my-temp-file.txt
                 TMP_STATE_TREE
+            ],
+            # Alternate root to test __env__ choices
+            'prod': [
+                os.path.join(FILES, 'file', 'prod'),
+                TMP_PRODENV_STATE_TREE
             ]
         }
         self.master_opts['ext_pillar'].append(
@@ -166,7 +216,11 @@ class TestDaemon(object):
                 )
             )}
         )
-        self.master_opts['extension_modules'] = os.path.join(INTEGRATION_TEST_DIR, 'files', 'extension_modules')
+
+        self.master_opts['extension_modules'] = os.path.join(
+            INTEGRATION_TEST_DIR, 'files', 'extension_modules'
+        )
+
         # clean up the old files
         self._clean()
 
@@ -197,7 +251,8 @@ class TestDaemon(object):
                     self.sub_minion_opts['sock_dir'],
                     self.minion_opts['sock_dir'],
                     TMP_STATE_TREE,
-                    TMP
+                    TMP_PRODENV_STATE_TREE,
+                    TMP,
                     ],
                    running_tests_user)
 
@@ -288,17 +343,15 @@ class TestDaemon(object):
         '''
         Kill the minion and master processes
         '''
-        import integration
-        integration.SYNDIC = None
-        self.sub_minion_process.terminate()
+        salt.master.clean_proc(self.sub_minion_process, wait_for_kill=50)
         self.sub_minion_process.join()
-        self.minion_process.terminate()
+        salt.master.clean_proc(self.minion_process, wait_for_kill=50)
         self.minion_process.join()
-        self.master_process.terminate()
+        salt.master.clean_proc(self.master_process, wait_for_kill=50)
         self.master_process.join()
-        self.syndic_process.terminate()
+        salt.master.clean_proc(self.syndic_process, wait_for_kill=50)
         self.syndic_process.join()
-        self.smaster_process.terminate()
+        salt.master.clean_proc(self.smaster_process, wait_for_kill=50)
         self.smaster_process.join()
         self._exit_mockbin()
         self._clean()
@@ -360,16 +413,18 @@ class TestDaemon(object):
 
         if sync_needed:
             # Wait for minions to "sync_all"
-            sync_minions = multiprocessing.Process(
-                target=self.sync_minion_modules,
-                args=(self.minion_targets, self.MINIONS_SYNC_TIMEOUT)
-            )
-            sync_minions.start()
-            sync_minions.join()
-            if sync_minions.exitcode > 0:
-                return False
-            sync_minions.terminate()
-            del sync_minions
+            for target in [self.sync_minion_modules,
+                           self.sync_minion_states]:
+                sync_minions = multiprocessing.Process(
+                    target=target,
+                    args=(self.minion_targets, self.MINIONS_SYNC_TIMEOUT)
+                )
+                sync_minions.start()
+                sync_minions.join()
+                if sync_minions.exitcode > 0:
+                    return False
+                sync_minions.terminate()
+                del sync_minions
 
         return True
 
@@ -406,8 +461,10 @@ class TestDaemon(object):
             shutil.rmtree(self.master_opts['root_dir'])
         if os.path.isdir(self.smaster_opts['root_dir']):
             shutil.rmtree(self.smaster_opts['root_dir'])
-        if os.path.isdir(TMP):
-            shutil.rmtree(TMP)
+
+        for dirname in (TMP, TMP_STATE_TREE, TMP_PRODENV_STATE_TREE):
+            if os.path.isdir(dirname):
+                shutil.rmtree(dirname)
 
     def wait_for_jid(self, targets, jid, timeout=120):
         time.sleep(1)  # Allow some time for minions to accept jobs
@@ -436,7 +493,7 @@ class TestDaemon(object):
                 sys.stdout.flush()
             time.sleep(1)
             now = datetime.now()
-        else:
+        else:  # pylint: disable=W0120
             sys.stdout.write(
                 '\n {RED_BOLD}*{ENDC} ERROR: Failed to get information '
                 'back\n'.format(**self.colors)
@@ -499,7 +556,7 @@ class TestDaemon(object):
 
             time.sleep(1)
             now = datetime.now()
-        else:
+        else:  # pylint: disable=W0120
             print(
                 '\n {RED_BOLD}*{ENDC} WARNING: Minions failed to connect '
                 'back. Tests requiring them WILL fail'.format(**self.colors)
@@ -507,26 +564,30 @@ class TestDaemon(object):
             print_header('=', sep='=', inline=True)
             raise SystemExit()
 
-    def sync_minion_modules(self, targets, timeout=120):
+    def sync_minion_modules_(self, modules_kind, targets, timeout=None):
+        if not timeout:
+            timeout = 120
         # Let's sync all connected minions
         print(
-            ' {LIGHT_BLUE}*{ENDC} Syncing minion\'s modules '
-            '(saltutil.sync_modules)'.format(
+            ' {LIGHT_BLUE}*{ENDC} Syncing minion\'s {1} '
+            '(saltutil.sync_{1})'.format(
                 ', '.join(targets),
+                modules_kind,
                 **self.colors
             )
         )
         syncing = set(targets)
         jid_info = self.client.run_job(
-            list(targets), 'saltutil.sync_modules',
+            list(targets), 'saltutil.sync_{0}'.format(modules_kind),
             expr_form='list',
             timeout=9999999999999999,
         )
 
         if self.wait_for_jid(targets, jid_info['jid'], timeout) is False:
             print(
-                ' {RED_BOLD}*{ENDC} WARNING: Minions failed to sync modules. '
-                'Tests requiring these modules WILL fail'.format(**self.colors)
+                ' {RED_BOLD}*{ENDC} WARNING: Minions failed to sync {0}. '
+                'Tests requiring these {0} WILL fail'.format(
+                    modules_kind, **self.colors)
             )
             raise SystemExit()
 
@@ -539,10 +600,23 @@ class TestDaemon(object):
                         syncing.remove(name)
                         continue
 
+                    if isinstance(output['ret'], salt._compat.string_types):
+                        # An errors has occurred
+                        print(
+                            ' {RED_BOLD}*{ENDC} {0} Failed so sync {2}: '
+                            '{1}'.format(
+                                name, output['ret'],
+                                modules_kind,
+                                **self.colors)
+                        )
+                        return False
+
                     print(
-                        '   {LIGHT_GREEN}*{ENDC} Synced {0} modules: '
+                        '   {LIGHT_GREEN}*{ENDC} Synced {0} {2}: '
                         '{1}'.format(
-                            name, ', '.join(output['ret']), **self.colors
+                            name,
+                            ', '.join(output['ret']),
+                            modules_kind, **self.colors
                         )
                     )
                     # Synced!
@@ -555,6 +629,12 @@ class TestDaemon(object):
                         )
         return True
 
+    def sync_minion_states(self, targets, timeout=None):
+        self.sync_minion_modules_('states', targets, timeout=timeout)
+
+    def sync_minion_modules(self, targets, timeout=None):
+        self.sync_minion_modules_('modules', targets, timeout=timeout)
+
 
 class AdaptedConfigurationTestCaseMixIn(object):
 
@@ -568,8 +648,12 @@ class AdaptedConfigurationTestCaseMixIn(object):
             # Running as root, the running user does not need to be updated
             return integration_config_dir
 
-        for fname in os.listdir(integration_config_dir):
-            self.get_config_file_path(fname)
+        for triplet in os.walk(integration_config_dir):
+            partial = triplet[0].replace(integration_config_dir, "")[1:]
+            for fname in triplet[2]:
+                if fname.startswith(('.', '_')):
+                    continue
+                self.get_config_file_path(os.path.join(partial, fname))
         return TMP_CONF_DIR
 
     def get_config_file_path(self, filename):
@@ -580,10 +664,11 @@ class AdaptedConfigurationTestCaseMixIn(object):
             # Running as root, the running user does not need to be updated
             return integration_config_file
 
-        if not os.path.isdir(TMP_CONF_DIR):
-            os.makedirs(TMP_CONF_DIR)
-
         updated_config_path = os.path.join(TMP_CONF_DIR, filename)
+        partial = os.path.dirname(updated_config_path)
+        if not os.path.isdir(partial):
+            os.makedirs(partial)
+
         if not os.path.isfile(updated_config_path):
             self.__update_config(integration_config_file, updated_config_path)
         return updated_config_path
@@ -647,13 +732,20 @@ class ModuleCase(TestCase, SaltClientTestCaseMixIn):
                     function, minion_tgt, orig
                 )
             )
+
+        # Try to match stalled state functions
+        orig[minion_tgt] = self._check_state_return(
+            orig[minion_tgt], func=function
+        )
+
         return orig[minion_tgt]
 
     def run_state(self, function, **kwargs):
         '''
         Run the state.single command and return the state return structure
         '''
-        return self.run_function('state.single', [function], **kwargs)
+        ret = self.run_function('state.single', [function], **kwargs)
+        return self._check_state_return(ret)
 
     @property
     def minion_opts(self):
@@ -682,6 +774,42 @@ class ModuleCase(TestCase, SaltClientTestCaseMixIn):
             self.get_config_file_path('master')
         )
 
+    def _check_state_return(self, ret, func='state.single'):
+        if isinstance(ret, dict):
+            # This is the supposed return format for state calls
+            return ret
+
+        if isinstance(ret, list):
+            jids = []
+            # These are usually errors
+            for item in ret[:]:
+                if not isinstance(item, salt._compat.string_types):
+                    # We don't know how to handle this
+                    continue
+                match = STATE_FUNCTION_RUNNING_RE.match(item)
+                if not match:
+                    # We don't know how to handle this
+                    continue
+                jid = match.group('jid')
+                if jid in jids:
+                    continue
+
+                jids.append(jid)
+
+                job_data = self.run_function(
+                    'saltutil.find_job', [jid]
+                )
+                job_kill = self.run_function('saltutil.kill_job', [jid])
+                msg = (
+                    'A running state.single was found causing a state lock. '
+                    'Job details: {0!r}  Killing Job Returned: {1!r}'.format(
+                        job_data, job_kill
+                    )
+                )
+                ret.append('[TEST SUITE ENFORCED]{0}'
+                           '[/TEST SUITE ENFORCED]'.format(msg))
+        return ret
+
 
 class SyndicCase(TestCase, SaltClientTestCaseMixIn):
     '''
@@ -712,28 +840,28 @@ class ShellCase(AdaptedConfigurationTestCaseMixIn, ShellTestCase):
     _script_dir_ = SCRIPT_DIR
     _python_executable_ = PYEXEC
 
-    def run_salt(self, arg_str, with_retcode=False):
+    def run_salt(self, arg_str, with_retcode=False, catch_stderr=False):
         '''
         Execute salt
         '''
         arg_str = '-c {0} {1}'.format(self.get_config_dir(), arg_str)
-        return self.run_script('salt', arg_str, with_retcode=with_retcode)
+        return self.run_script('salt', arg_str, with_retcode=with_retcode, catch_stderr=catch_stderr)
 
-    def run_run(self, arg_str, with_retcode=False):
+    def run_run(self, arg_str, with_retcode=False, catch_stderr=False):
         '''
         Execute salt-run
         '''
         arg_str = '-c {0} {1}'.format(self.get_config_dir(), arg_str)
-        return self.run_script('salt-run', arg_str, with_retcode=with_retcode)
+        return self.run_script('salt-run', arg_str, with_retcode=with_retcode, catch_stderr=catch_stderr)
 
-    def run_run_plus(self, fun, options='', *arg):
+    def run_run_plus(self, fun, options='', *arg, **kwargs):
         '''
         Execute Salt run and the salt run function and return the data from
         each in a dict
         '''
         ret = {}
         ret['out'] = self.run_run(
-            '{0} {1} {2}'.format(options, fun, ' '.join(arg))
+            '{0} {1} {2}'.format(options, fun, ' '.join(arg)), catch_stderr=kwargs.get('catch_stderr', None)
         )
         opts = salt.config.master_config(
             self.get_config_file_path('master')
@@ -756,16 +884,23 @@ class ShellCase(AdaptedConfigurationTestCaseMixIn, ShellTestCase):
             with_retcode=with_retcode
         )
 
-    def run_cp(self, arg_str, with_retcode=False):
+    def run_cp(self, arg_str, with_retcode=False, catch_stderr=False):
         '''
         Execute salt-cp
         '''
         arg_str = '--config-dir {0} {1}'.format(self.get_config_dir(), arg_str)
-        return self.run_script('salt-cp', arg_str, with_retcode=with_retcode)
+        return self.run_script('salt-cp', arg_str, with_retcode=with_retcode, catch_stderr=catch_stderr)
 
-    def run_call(self, arg_str, with_retcode=False):
+    def run_call(self, arg_str, with_retcode=False, catch_stderr=False):
         arg_str = '--config-dir {0} {1}'.format(self.get_config_dir(), arg_str)
-        return self.run_script('salt-call', arg_str, with_retcode=with_retcode)
+        return self.run_script('salt-call', arg_str, with_retcode=with_retcode, catch_stderr=catch_stderr)
+
+    def run_cloud(self, arg_str, catch_stderr=False, timeout=None):
+        '''
+        Execute salt-cloud
+        '''
+        arg_str = '-c {0} {1}'.format(self.get_config_dir(), arg_str)
+        return self.run_script('salt-cloud', arg_str, catch_stderr, timeout)
 
 
 class ShellCaseCommonTestsMixIn(CheckShellBinaryNameAndVersionMixIn):
@@ -976,3 +1111,21 @@ class SaltReturnAssertsMixIn(object):
         return self.assertNotEqual(
             self.__getWithinSaltReturn(ret, keys), comparison
         )
+
+
+class ClientCase(AdaptedConfigurationTestCaseMixIn, TestCase):
+    '''
+    A base class containing relevant options for starting the various Salt
+    Python API entrypoints
+    '''
+    def get_opts(self):
+        return salt.config.client_config(self.get_config_file_path('master'))
+
+    def mkdir_p(self, path):
+        try:
+            os.makedirs(path)
+        except OSError as exc:  # Python >2.5
+            if exc.errno == errno.EEXIST and os.path.isdir(path):
+                pass
+            else:
+                raise
