@@ -6,6 +6,7 @@ import tempfile
 import json
 import datetime
 import textwrap
+import pprint
 
 # Import Salt Testing libs
 from salttesting import skipIf, TestCase
@@ -44,22 +45,22 @@ class MockFileClient(object):
             loader._file_client = self
         self.requests = []
 
-    def get_file(self, template, dest='', makedirs=False, env='base'):
+    def get_file(self, template, dest='', makedirs=False, saltenv='base'):
         self.requests.append({
             'path': template,
             'dest': dest,
             'makedirs': makedirs,
-            'env': env
+            'saltenv': saltenv
         })
 
 
 class TestSaltCacheLoader(TestCase):
     def test_searchpath(self):
         '''
-        The searchpath is based on the cachedir option and the env parameter
+        The searchpath is based on the cachedir option and the saltenv parameter
         '''
         tmp = tempfile.gettempdir()
-        loader = SaltCacheLoader({'cachedir': tmp}, env='test')
+        loader = SaltCacheLoader({'cachedir': tmp}, saltenv='test')
         assert loader.searchpath == [os.path.join(tmp, 'files', 'test')]
 
     def test_mockclient(self):
@@ -79,7 +80,7 @@ class TestSaltCacheLoader(TestCase):
         assert len(fc.requests)
         self.assertEqual(fc.requests[0]['path'], 'salt://hello_simple')
 
-    def get_test_env(self):
+    def get_test_saltenv(self):
         '''
         Setup a simple jinja test environment
         '''
@@ -92,7 +93,7 @@ class TestSaltCacheLoader(TestCase):
         '''
         You can import and use macros from other files
         '''
-        fc, jinja = self.get_test_env()
+        fc, jinja = self.get_test_saltenv()
         result = jinja.get_template('hello_import').render()
         self.assertEqual(result, 'Hey world !a b !')
         assert len(fc.requests) == 2
@@ -103,7 +104,7 @@ class TestSaltCacheLoader(TestCase):
         '''
         You can also include a template that imports and uses macros
         '''
-        fc, jinja = self.get_test_env()
+        fc, jinja = self.get_test_saltenv()
         result = jinja.get_template('hello_include').render()
         self.assertEqual(result, 'Hey world !a b !')
         assert len(fc.requests) == 3
@@ -115,7 +116,7 @@ class TestSaltCacheLoader(TestCase):
         '''
         Context variables are passes to the included template by default.
         '''
-        _, jinja = self.get_test_env()
+        _, jinja = self.get_test_saltenv()
         result = jinja.get_template('hello_include').render(a='Hi', b='Salt')
         self.assertEqual(result, 'Hey world !Hi Salt !')
 
@@ -140,7 +141,7 @@ class TestGetTemplate(TestCase):
         with salt.utils.fopen(fn_) as fp_:
             out = render_jinja_tmpl(
                     fp_.read(),
-                    dict(opts=self.local_opts, env='other'))
+                    dict(opts=self.local_opts, saltenv='other'))
         self.assertEqual(out, 'world\n')
 
     def test_fallback_noloader(self):
@@ -151,10 +152,10 @@ class TestGetTemplate(TestCase):
         filename = os.path.join(TEMPLATES_DIR, 'files', 'test', 'hello_import')
         out = render_jinja_tmpl(
                 salt.utils.fopen(filename).read(),
-                dict(opts=self.local_opts, env='other'))
+                dict(opts=self.local_opts, saltenv='other'))
         self.assertEqual(out, 'Hey world !a b !\n')
 
-    def test_env(self):
+    def test_saltenv(self):
         '''
         If the template is within the searchpath it can
         import, include and extend other templates.
@@ -169,11 +170,86 @@ class TestGetTemplate(TestCase):
         out = render_jinja_tmpl(
                 salt.utils.fopen(filename).read(),
                 dict(opts={'cachedir': TEMPLATES_DIR, 'file_client': 'remote'},
-                     a='Hi', b='Salt', env='test'))
+                     a='Hi', b='Salt', saltenv='test'))
         self.assertEqual(out, 'Hey world !Hi Salt !\n')
         self.assertEqual(fc.requests[0]['path'], 'salt://macro')
         SaltCacheLoader.file_client = _fc
 
+    def test_macro_additional_log_for_generalexc(self):
+        '''
+        If we failed in a macro because of eg a typeerror, get
+        more output from trace.
+        '''
+        expected = r'''Jinja error:.*division.*
+.*/macrogeneral\(2\):
+---
+\{% macro mymacro\(\) -%\}
+\{\{ 1/0 \}\}    <======================
+\{%- endmacro %\}
+---.*'''
+        filename = os.path.join(TEMPLATES_DIR,
+                                'files', 'test', 'hello_import_generalerror')
+        fc = MockFileClient()
+        _fc = SaltCacheLoader.file_client
+        SaltCacheLoader.file_client = lambda loader: fc
+        self.assertRaisesRegexp(
+            SaltRenderError,
+            expected,
+            render_jinja_tmpl,
+            salt.utils.fopen(filename).read(),
+            dict(opts=self.local_opts, saltenv='other'))
+        SaltCacheLoader.file_client = _fc
+
+    def test_macro_additional_log_for_undefined(self):
+        '''
+        If we failed in a macro because of undefined variables, get
+        more output from trace.
+        '''
+        expected = r'''Jinja variable 'b' is undefined
+.*/macroundefined\(2\):
+---
+\{% macro mymacro\(\) -%\}
+\{\{b.greetee\}\} <-- error is here    <======================
+\{%- endmacro %\}
+---'''
+        filename = os.path.join(TEMPLATES_DIR,
+                                'files', 'test', 'hello_import_undefined')
+        fc = MockFileClient()
+        _fc = SaltCacheLoader.file_client
+        SaltCacheLoader.file_client = lambda loader: fc
+        self.assertRaisesRegexp(
+            SaltRenderError,
+            expected,
+            render_jinja_tmpl,
+            salt.utils.fopen(filename).read(),
+            dict(opts=self.local_opts, saltenv='other'))
+        SaltCacheLoader.file_client = _fc
+
+    def test_macro_additional_log_syntaxerror(self):
+        '''
+        If  we failed in a macro, get more output from trace.
+        '''
+        expected = r'''Jinja syntax error: expected token .*end.*got '-'.*
+.*/macroerror\(2\):
+---
+# macro
+\{% macro mymacro\(greeting, greetee='world'\) -\} <-- error is here    <======================
+\{\{ greeting ~ ' ' ~ greetee \}\} !
+\{%- endmacro %\}
+---.*'''
+        filename = os.path.join(TEMPLATES_DIR,
+                                'files', 'test', 'hello_import_error')
+        fc = MockFileClient()
+        _fc = SaltCacheLoader.file_client
+        SaltCacheLoader.file_client = lambda loader: fc
+        self.assertRaisesRegexp(
+            SaltRenderError,
+            expected,
+            render_jinja_tmpl,
+            salt.utils.fopen(filename).read(),
+            dict(opts=self.local_opts, saltenv='other'))
+        SaltCacheLoader.file_client = _fc
+
     def test_non_ascii_encoding(self):
         fc = MockFileClient()
         # monkey patch file client
@@ -183,7 +259,7 @@ class TestGetTemplate(TestCase):
         out = render_jinja_tmpl(
                 salt.utils.fopen(filename).read(),
                 dict(opts={'cachedir': TEMPLATES_DIR, 'file_client': 'remote'},
-                     a='Hi', b='Sàlt', env='test'))
+                     a='Hi', b='Sàlt', saltenv='test'))
         self.assertEqual(out, u'Hey world !Hi Sàlt !\n')
         self.assertEqual(fc.requests[0]['path'], 'salt://macro')
         SaltCacheLoader.file_client = _fc
@@ -194,22 +270,15 @@ class TestGetTemplate(TestCase):
         out = render_jinja_tmpl(
                 salt.utils.fopen(filename).read(),
                 dict(opts={'cachedir': TEMPLATES_DIR, 'file_client': 'remote'},
-                     a='Hi', b='Sàlt', env='test'))
+                     a='Hi', b='Sàlt', saltenv='test'))
         self.assertEqual(u'Assunção\n', out)
         self.assertEqual(fc.requests[0]['path'], 'salt://macro')
         SaltCacheLoader.file_client = _fc
 
-    def test_non_ascii(self):
-        fn = os.path.join(TEMPLATES_DIR, 'files', 'test', 'non_ascii')
-        out = JINJA(fn, opts=self.local_opts, env='other')
-        with salt.utils.fopen(out['data']) as fp:
-            result = fp.read().decode('utf-8')
-            self.assertEqual(u'Assunção\n', result)
-
     @skipIf(HAS_TIMELIB is False, 'The `timelib` library is not installed.')
     def test_strftime(self):
         response = render_jinja_tmpl('{{ "2002/12/25"|strftime }}',
-                dict(opts=self.local_opts, env='other'))
+                dict(opts=self.local_opts, saltenv='other'))
         self.assertEqual(response, '2002-12-25')
 
         objects = (
@@ -221,20 +290,20 @@ class TestGetTemplate(TestCase):
 
         for object in objects:
             response = render_jinja_tmpl('{{ object|strftime }}',
-                    dict(object=object, opts=self.local_opts, env='other'))
+                    dict(object=object, opts=self.local_opts, saltenv='other'))
             self.assertEqual(response, '2002-12-25')
 
             response = render_jinja_tmpl('{{ object|strftime("%b %d, %Y") }}',
-                    dict(object=object, opts=self.local_opts, env='other'))
+                    dict(object=object, opts=self.local_opts, saltenv='other'))
             self.assertEqual(response, 'Dec 25, 2002')
 
             response = render_jinja_tmpl('{{ object|strftime("%y") }}',
-                    dict(object=object, opts=self.local_opts, env='other'))
+                    dict(object=object, opts=self.local_opts, saltenv='other'))
             self.assertEqual(response, '02')
 
     def test_non_ascii(self):
         fn = os.path.join(TEMPLATES_DIR, 'files', 'test', 'non_ascii')
-        out = JINJA(fn, opts=self.local_opts, env='other')
+        out = JINJA(fn, opts=self.local_opts, saltenv='other')
         with salt.utils.fopen(out['data']) as fp:
             result = fp.read().decode('utf-8')
             self.assertEqual(u'Assunção\n', result)
@@ -277,7 +346,29 @@ class TestGetTemplate(TestCase):
             expected,
             render_jinja_tmpl,
             template,
-            dict(opts=self.local_opts, env='other')
+            dict(opts=self.local_opts, saltenv='other')
+        )
+
+    def test_render_with_unicode_syntax_error(self):
+        template = u'hello\n\n{{ bad\n\nfoo\ud55c'
+        expected = r'.*---\nhello\n\n{{ bad\n\nfoo\xed\x95\x9c    <======================\n---'
+        self.assertRaisesRegexp(
+            SaltRenderError,
+            expected,
+            render_jinja_tmpl,
+            template,
+            dict(opts=self.local_opts, saltenv='other')
+        )
+
+    def test_render_with_utf8_syntax_error(self):
+        template = 'hello\n\n{{ bad\n\nfoo\xed\x95\x9c'
+        expected = r'.*---\nhello\n\n{{ bad\n\nfoo\xed\x95\x9c    <======================\n---'
+        self.assertRaisesRegexp(
+            SaltRenderError,
+            expected,
+            render_jinja_tmpl,
+            template,
+            dict(opts=self.local_opts, saltenv='other')
         )
 
     def test_render_with_undefined_variable(self):
@@ -288,7 +379,29 @@ class TestGetTemplate(TestCase):
             expected,
             render_jinja_tmpl,
             template,
-            dict(opts=self.local_opts, env='other')
+            dict(opts=self.local_opts, saltenv='other')
+        )
+
+    def test_render_with_undefined_variable_utf8(self):
+        template = "hello\xed\x95\x9c\n\n{{ foo }}\n\nfoo"
+        expected = r'Jinja variable \'foo\' is undefined;.*\n\n---\nhello\xed\x95\x9c\n\n{{ foo }}.*'
+        self.assertRaisesRegexp(
+            SaltRenderError,
+            expected,
+            render_jinja_tmpl,
+            template,
+            dict(opts=self.local_opts, saltenv='other')
+        )
+
+    def test_render_with_undefined_variable_unicode(self):
+        template = u"hello\ud55c\n\n{{ foo }}\n\nfoo"
+        expected = r'Jinja variable \'foo\' is undefined;.*\n\n---\nhello\xed\x95\x9c\n\n{{ foo }}.*'
+        self.assertRaisesRegexp(
+            SaltRenderError,
+            expected,
+            render_jinja_tmpl,
+            template,
+            dict(opts=self.local_opts, saltenv='other')
         )
 
 
@@ -315,6 +428,17 @@ class TestCustomExtensions(TestCase):
         rendered = env.from_string('{{ dataset|yaml }}').render(dataset=dataset)
         self.assertEqual(dataset, yaml.load(rendered))
 
+    def test_serialize_python(self):
+        dataset = {
+            "foo": True,
+            "bar": 42,
+            "baz": [1, 2, 3],
+            "qux": 2.0
+        }
+        env = Environment(extensions=[SerializerExtension])
+        rendered = env.from_string('{{ dataset|python }}').render(dataset=dataset)
+        self.assertEqual(rendered, pprint.pformat(dataset))
+
     def test_load_yaml(self):
         env = Environment(extensions=[SerializerExtension])
         rendered = env.from_string('{% set document = "{foo: it works}"|load_yaml %}{{ document.foo }}').render()
@@ -473,7 +597,6 @@ class TestCustomExtensions(TestCase):
     #     return
 
 
-
 if __name__ == '__main__':
     from integration import run_tests
     run_tests(TestSaltCacheLoader, TestGetTemplate, TestCustomExtensions,
