@@ -1,3 +1,5 @@
+# -*- coding: utf-8 -*-
+
 # Import python libs
 import os
 import sys
@@ -79,8 +81,6 @@ class TestFind(TestCase):
         self.assertEqual(resolution, 86400)
         self.assertEqual(modifier, '-')
 
-
-
     def test_parse_size(self):
         self.assertRaises(ValueError, salt.utils.find._parse_size, '')
         self.assertRaises(ValueError, salt.utils.find._parse_size, '1s1s')
@@ -145,7 +145,7 @@ class TestFind(TestCase):
             ValueError, salt.utils.find.RegexOption, 'name', '(.*}'
         )
 
-        option = salt.utils.find.RegexOption('name', '.*\.txt')
+        option = salt.utils.find.RegexOption('name', r'.*\.txt')
         self.assertIs(option.match('', '', ''), None)
         self.assertIs(option.match('', 'hello.txt', '').group(), 'hello.txt')
         self.assertIs(option.match('', 'HELLO.TXT', ''), None)
@@ -155,7 +155,7 @@ class TestFind(TestCase):
             ValueError, salt.utils.find.IregexOption, 'name', '(.*}'
         )
 
-        option = salt.utils.find.IregexOption('name', '.*\.txt')
+        option = salt.utils.find.IregexOption('name', r'.*\.txt')
         self.assertIs(option.match('', '', ''), None)
         self.assertIs(option.match('', 'hello.txt', '').group(), 'hello.txt')
         self.assertIs(option.match('', 'HELLO.TXT', '').group(), 'HELLO.TXT')
@@ -476,7 +476,7 @@ class TestFinder(TestCase):
             str(finder.criteria[0].__class__)[-13:-2], 'InameOption'
         )
 
-        finder = salt.utils.find.Finder({'regex': '.*\.txt'})
+        finder = salt.utils.find.Finder({'regex': r'.*\.txt'})
         self.assertEqual(
             str(finder.actions[0].__class__)[-13:-2], 'PrintOption'
         )
@@ -484,7 +484,7 @@ class TestFinder(TestCase):
             str(finder.criteria[0].__class__)[-13:-2], 'RegexOption'
         )
 
-        finder = salt.utils.find.Finder({'iregex': '.*\.txt'})
+        finder = salt.utils.find.Finder({'iregex': r'.*\.txt'})
         self.assertEqual(
             str(finder.actions[0].__class__)[-13:-2], 'PrintOption'
         )
